__all__ = ["POI_CATG_DICT"]

POI_CATG_DICT = {
    "outdoor_activity": [
        "bandstand",
        "beach_resort",
        "bird_hide",
        "bleachers",
        "firepit",
        "fishing",
        "garden",
        "nature_reserve",
        "park",
        "picnic_table",
        "playground",
        "resort",
        "summer_camp",
        "swimming_area",
        "water_park",
        "wildlife_hide",
    ],
    "indoor_entertainment": [
        "adult_gaming_centre",
        "amusement_arcade",
        "bowling_alley",
        "disc_golf_course",
        "escape_game",
        "fitness_centre",
        "fitness_station",
        "golf_course",
        "miniature_golf",
        "sauna",
        "tanning_salon",
        "trampoline_park",
        "bar",
        "biergarten",
        "cafe",
        "fast_food",
        "food_court",
        "ice_cream",
        "pub",
        "restaurant",
        "arts_centre",
        "brothel",
        "casino",
        "cinema",
        "community_centre",
        "conference_centre",
        "events_venue",
        "exhibition_centre",
        "fountain",
        "gambling",
        "love_hotel",
        "music_venue",
        "nightclub",
        "planetarium",
        "public_bookcase",
        "social_centre",
        "stage",
        "stripclub",
        "studio",
        "swingerclub",
        "theatre",
    ],
    "sports_facility": [
        "horse_riding",
        "ice_rink",
        "marina",
        "pitch",
        "sports_centre",
        "sports_hall",
        "stadium",
        "track",
        "swimming_pool",
    ],
    "water_activity": [
        "beach_resort",
        "ice_rink",
        "marina",
        "slipway",
        "swimming_area",
        "swimming_pool",
        "water_park",
    ],
    "nature_and_wildlife_observation": [
        "bird_hide",
        "nature_reserve",
        "wildlife_hide",
        "hunting_stand",
    ],
    "children_playground": ["playground", "summer_camp", "miniature_golf", "dog_park"],
    "restaurant": [
        "bar",
        "biergarten",
        "cafe",
        "fast_food",
        "food_court",
        "ice_cream",
        "pub",
        "restaurant",
    ],
    "education_institution": [
        "college",
        "dancing_school",
        "driving_school",
        "first_aid_school",
        "kindergarten",
        "language_school",
        "library",
        "surf_school",
        "toy_library",
        "research_institute",
        "training",
        "music_school",
        "school",
        "traffic_park",
        "university",
    ],
    "transportation_facility": [
        "bicycle_parking",
        "bicycle_repair_station",
        "bicycle_rental",
        "bicycle_wash",
        "boat_rental",
        "boat_sharing",
        "bus_station",
        "car_rental",
        "car_sharing",
        "car_wash",
        "compressed_air",
        "vehicle_inspection",
        "charging_station",
        "driver_training",
        "ferry_terminal",
        "fuel",
        "grit_bin",
        "motorcycle_parking",
        "parking",
        "parking_entrance",
        "parking_space",
        "taxi",
        "weighbridge",
    ],
    "financial_service": [
        "atm",
        "payment_terminal",
        "bank",
        "bureau_de_change",
        "money_transfer",
        "payment_centre",
    ],
    "medical_care": [
        "baby_hatch",
        "clinic",
        "dentist",
        "doctors",
        "hospital",
        "nursing_home",
        "pharmacy",
        "social_facility",
        "veterinary",
    ],
    "cultural_and_artistic": [
        "arts_centre",
        "brothel",
        "casino",
        "cinema",
        "community_centre",
        "conference_centre",
        "events_venue",
        "exhibition_centre",
        "fountain",
        "gambling",
        "love_hotel",
        "music_venue",
        "nightclub",
        "planetarium",
        "public_bookcase",
        "social_centre",
        "stage",
        "stripclub",
        "studio",
        "swingerclub",
        "theatre",
    ],
    "public_service": [
        "bbq",
        "bench",
        "dog_toilet",
        "dressing_room",
        "drinking_water",
        "give_box",
        "lounge",
        "mailroom",
        "parcel_locker",
        "shelter",
        "shower",
        "telephone",
        "toilets",
        "water_point",
        "watering_place",
        "sanitary_dump_station",
        "recycling",
        "waste_basket",
        "waste_disposal",
        "waste_transfer_station",
        "post_box",
        "post_depot",
        "post_office",
        "courthouse",
        "fire_station",
        "police",
        "prison",
        "ranger_station",
        "townhall",
    ],
    "other_special_purpose": [
        "animal_boarding",
        "animal_breeding",
        "animal_shelter",
        "animal_training",
        "baking_oven",
        "clock",
        "crematorium",
        "dive_centre",
        "funeral_hall",
        "grave_yard",
        "hunting_stand",
        "internet_cafe",
        "kitchen",
        "kneipp_water_cure",
        "lounger",
        "marketplace",
        "monastery",
        "mortuary",
        "photo_booth",
        "place_of_mourning",
        "place_of_worship",
        "public_bath",
        "public_building",
        "refugee_site",
        "vending_machine",
    ],
}
