"""
AgentSociety2 自定义模块包

此包用于存放用户自定义的 Agent 和环境模块代码。

目录结构:
- agents/    : 自定义 Agent 类
- envs/      : 自定义环境模块类

使用方法:
1. 在相应目录创建 .py 文件
2. 继承 AgentBase 或 EnvBase
3. 运行 VSCode 命令 "扫描自定义模块"
"""

__all__ = []
